
global w_bar, weightfolder, savedweight, trainloader, valloader, testloader
global sf, ch, tp, net,epochs, lr, sub, netname, stop_thread, val_ratio, n_cuda, n_class, bs
global ypred, ypr, acc
global montage, electrode
global x_train, y_train, x_test , y_test 

x_train=0
y_train=0
x_test=0
y_test=0 

w_bar= 0
weightfolder= 0
savedweight=0 

trainloader= 0
valloader =0
testloader= 0
val_ratio=0

n_cuda=0
n_class=0
epochs=0
lr=0
sub=0
netname=0 
stop_thread= 0
ypred= 0
ypr= 0
acc=0
net=0
sf=0
ch=0
tp=0
bs=0

montage=0
electrode=0